<?php
require_once 'DB.php';	// PEAR:DB
ini_set("session.save_handler", "user");
/**
 * SessionDBクラス
 *
 * PEAR::DBをインストールし、SessionDBをrequireすれば、普通にSession関数を使うだけで使える。
 *
 * 使い方
 * require_once 'SessionDB.php';
 * session_start();
 * $_SESSION['id'] = 1;
 * print $_SESSION['id'];
 * // session_destroy();
 *
 * create table sessions (
 * id int(11) not null auto_increment,
 * sessid varchar(32) not null,
 * data text,
 * modified datetime,
 * primary key (id),
 * unique idx_sessid(sessid)
 * );
 */
class SessionDB
{
	/**#@+
	 * @access	protected
	 */
	static $_sqltype;// = "mysql";
	static $_host;// = "localhost";
	static $_user;// = "root";
	static $_pass;// = "pass";
	static $_name;// = "SessionDB";
	static $_table;// = "sessions";
	/**#@-*/
	
	/**#@+
	 * @access	public
	 */
	/**
	 * DB接続
	 */
	function &connect()
	{
		$dsn = self::getDsn();
		return DB::connect($dsn);
	}
	/**
	 * DSN取得
	 */
	function &getDsn()
	{
		return sprintf("%s://%s:%s@%s/%s", self::$_sqltype, self::$_user, self::$_pass, self::$_host, self::$_name);
	}
	/**
	 * DB切断
	 */
	function &disconnect($db)
	{
		$db->disconnect();
	}
	/**
	 * セッションオープン時
	 */
	function open()
	{
	    return true;
	}
	/**
	 * セッションクローズ時
	 */
	function close()
	{
	    return true;
	}
	/**
	 * セッション読み込み時
	 */
	function read($id)
	{
	    $ret = '';
	    $db =& self::connect();
		if (DB::isError($db)) die($db->getMessage());    // メッセージ表示
	    $sql  = sprintf("select data from %s where sessid = %s;", self::$_table, $db->quote($id));
	    $ret = $db->getOne($sql);
		self::disconnect(&$db);
	    return $ret;
	}
	/**
	 * セッション書き込み時
	 */
	function write($id, $data)
	{
	    $db =& self::connect();
	    $sql  = sprintf("replace into %s (sessid, data, modified) values  (%s, %s, now());", self::$_table, $db->quote($id), $db->quote($data));
	    $rs = $db->query($sql);
		self::disconnect(&$db);
	    return true;
	}
	/**
	 * セッション破棄時
	 */
	function destroy($id)
	{
	    $db =& self::connect();
	    $sql  = sprintf("delete from %s where sessid = %s;", self::$_table, $db->quote($id));
	    $rs = $db->query($sql);
		self::disconnect(&$db);
	    return true;
	}
	/**
	 * ガーベージコレクション時
	 */
	function clean($maxlifetime)
	{
	    $db =& self::connect();
	    $sql  = sprintf("delete from %s where modified < current_timestamp + '-%d secs';", self::$_table, (int)$maxlifetime);
	    $rs = $db->query($sql);
		self::disconnect(&$db);
	    return true;
	}
	/**#@-*/
}
// session_set_save_handler(array('SessionDB','open'), array('SessionDB','close'), array('SessionDB','read'), array('SessionDB','write'), array('SessionDB','destroy'), array('SessionDB','clean'));
?>